/**
	$Id: AbstractOnSaveAction.java 5490 2010-11-15 12:53:05Z nicklas $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.edit;

import net.sf.basedb.clients.web.extensions.JspContext;

/**
	Abstract base implementation of the OnSaveAction interface.
	This contains empty method stubs for all interface methods. 
	It is recommended for subclasses to extend this class to
	avoid problems in the future if new methods are added to
	the interface.
	
	@author Nicklas
	@since 2.17
	@base.modified $Date: 2010-11-15 13:53:05 +0100 (Mon, 15 Nov 2010) $
*/
public abstract class AbstractOnSaveAction
	implements OnSaveAction
{

	private JspContext jspContext;

	/**
		Create a on-save action without a context
	*/
	protected AbstractOnSaveAction()
	{}
	
	/**
		Create a on-save action with a jsp context.
	*/
	protected AbstractOnSaveAction(JspContext jspContext)
	{
		this.jspContext = jspContext;
	}
	
	/*
		From the OnSaveAction interface
		-------------------------------
	*/
	@Override
	public void onSave()
	{}
	
	@Override
	public void onCommit()
	{}

	@Override
	public void onRollback(Exception ex)
	{}
	// -------------------------------
	
	/**
		Get the JSP context of the current invokation.
		Can be null if no context has been set in the constructor or
		the setter method.
	*/
	protected void setJspContext(JspContext jspContext)
	{
		this.jspContext = jspContext;
	}
	
	/**
		Set the JSP context of the current invokation.
	*/
	protected JspContext getJspContext()
	{
		return jspContext;
	}
	
}
