/**
	$Id$

	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.list;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.Type;
import net.sf.basedb.util.Enumeration;
import net.sf.basedb.util.formatter.Formatter;

/**
	A simple bean-like implementation of the ListColumn interface.
	This class is abstract and subclasses must implement the 
	{@link #getValue(DbControl, Object)} method. The default implementation 
	of the {@link #getExportValue(DbControl, Object)}
	simply copies the <code>getValue()</code> result. A subclass that need different
	values must override the <code>getExportValue()</code> method as well.
	
	@param <I> The type of items that are listed
	@param <V> The type of value that is returned by the getValue() method
	@author Nicklas
	@since 3.2
*/
public abstract class AbstractListColumnBean<I, V>
	implements ListColumnAction<I, V>
{

	
	private String id;
	private String clazz;
	private String style;
	private String title;
	private String subtitle;
	private String cellClass;
	private String cellStyle;
	private boolean disableOverflowCheck;
	private String tooltip;
	private String property;
	private Type valueType;
	private boolean sortable;
	private String sortProperty;
	private boolean filterable;
	private String filterProperty;
	private boolean exportable;
	private String exportProperty;
	private Formatter<?> exportFormatter;
	private Enumeration<String, String> enumeration;
	private Formatter<? super V> formatter;
	
	public AbstractListColumnBean()
	{}

	/*
		From the ListColumnAction interface
		-----------------------------------
	*/
	@Override
	public String getId() 
	{
		return id;
	}
	public void setId(String id)
	{
		this.id = id;
	}
	
	@Override
	public String getClazz() 
	{
		return clazz;
	}
	public void setClazz(String clazz)
	{
		this.clazz = clazz;
	}

	@Override
	public String getStyle() 
	{
		return style;
	}
	public void setStyle(String style)
	{
		this.style = style;
	}
	
	@Override
	public String getTitle() 
	{
		return title;
	}
	public void setTitle(String title)
	{
		this.title = title;
	}

	@Override
	public String getSubtitle() 
	{
		return subtitle;
	}
	public void setSubtitle(String subtitle)
	{
		this.subtitle = subtitle;
	}

	@Override
	public String getCellClass() 
	{
		return cellClass;
	}
	public void setCellClass(String cellClass)
	{
		this.cellClass = cellClass;
	}

	@Override
	public String getCellStyle() 
	{
		return cellStyle;
	}
	public void setCellStyle(String cellStyle)
	{
		this.cellStyle = cellStyle;
	}

	@Override
	public boolean getDisableOverflowCheck() 
	{
		return disableOverflowCheck;
	}
	public void setDisableOverflowCheck(boolean disableOverflowCheck)
	{
		this.disableOverflowCheck = disableOverflowCheck;
	}

	
	@Override
	public String getTooltip() 
	{
		return tooltip;
	}
	public void setTooltip(String tooltip)
	{
		this.tooltip = tooltip;
	}

	@Override
	public String getProperty() 
	{
		return property;
	}
	public void setProperty(String property)
	{
		this.property = property;
	}

	@Override
	public Type getValueType() 
	{
		return valueType;
	}
	public void setValueType(Type valueType)
	{
		this.valueType = valueType;
	}

	@Override
	public boolean isSortable() 
	{
		return sortable;
	}
	public void setSortable(boolean sortable)
	{
		this.sortable = sortable;
	}

	@Override
	public String getSortProperty() 
	{
		return sortProperty;
	}
	public void setSortProperty(String sortProperty)
	{
		this.sortProperty = sortProperty;
	}

	@Override
	public boolean isFilterable() 
	{
		return filterable;
	}
	public void setFilterable(boolean filterable)
	{
		this.filterable = filterable;
	}

	@Override
	public String getFilterProperty() 
	{
		return filterProperty;
	}
	public void setFilterProperty(String filterProperty)
	{
		this.filterProperty = filterProperty;
	}

	@Override
	public boolean isExportable() 
	{
		return exportable;
	}
	public void setExportable(boolean exportable)
	{
		this.exportable = exportable;
	}

	@Override
	public String getExportProperty() 
	{
		return exportProperty;
	}
	public void setExportProperty(String exportProperty)
	{
		this.exportProperty = exportProperty;
	}

	@Override
	public Enumeration<String, String> getEnumeration() 
	{
		return enumeration;
	}
	public void setEnumeration(Enumeration<String, String> enumeration)
	{
		this.enumeration = enumeration;
	}
	
	@Override
	public Formatter<? super V> getFormatter() 
	{
		return formatter;
	}
	public void setFormatter(Formatter<? super V> formatter)
	{
		this.formatter = formatter;
	}
	
	@Override
	public Formatter<?> getExportFormatter()
	{
		return exportFormatter;
	}
	public void setExportFormatter(Formatter<?> exportFormatter)
	{
		this.exportFormatter = exportFormatter;
	}
	
	/**
		This implementation simply forward to {@link #getValue(DbControl, Object)}.
		If the exported value should be different from the regular value the
		subclass must override this method as well.
	*/
	@Override
	public Object getExportValue(DbControl dc, I item)
	{
		return getValue(dc, item);
	}
	// -----------------------------------------
	
	
}
