package net.sf.basedb.clients.web.extensions.list;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.ItemList;
import net.sf.basedb.core.RawBioAssay;
import net.sf.basedb.core.RawDataType;
import net.sf.basedb.util.filter.Filter;

/**
	Filter implementation that accept raw bioassay items of a given raw data type.
	The filter can also be used to accept Item lists with a matching 
	{@link ItemList#getRawDataType()}
	(or without raw data type if the member type matches).

	@author nicklas
	@since 3.17
*/
public class RawDataTypeFilter 
	implements Filter<Annotatable>
{
	private final RawDataType rdt;
	
	public RawDataTypeFilter(RawDataType rdt) 
	{
		this.rdt = rdt;
	}
	
	@Override
	public boolean evaluate(Annotatable item) 
	{
		Item itemType = item.getType();
		if (itemType == Item.ITEMLIST)
		{
			ItemList list = (ItemList)item;
			RawDataType memberRawdataType = list.getRawDataType();
			return (list.getMemberType() == Item.RAWBIOASSAY) && (memberRawdataType == null || rdt.equals(memberRawdataType));
		}
		else if (itemType == Item.RAWBIOASSAY) 
		{
			return rdt.equals(((RawBioAssay)item).getRawDataType());
		}
		return false;
	}
}
