package net.sf.basedb.clients.web.extensions.list;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import net.sf.basedb.core.Annotatable;
import net.sf.basedb.core.Item;
import net.sf.basedb.core.Metadata;
import net.sf.basedb.core.Metadata.PropertyPath;
import net.sf.basedb.core.SyncFilter.SourceItemTransform;
import net.sf.basedb.core.query.Restriction;

/**
	Related item column that get values from parent or child items by specifying
	a "property path". See {@link Metadata}.

	@author nicklas
	@since 3.16, 3.17
*/
public class RelatedItemPropertyColumn 
	extends RelatedItemColumn
{

	private final PropertyPath<? super Annotatable, ?> property;
	
	RelatedItemPropertyColumn(int index, String id, Item sourceType, Item targetType, SourceItemTransform direction, Restriction targetRestriction, RelatedItemHelper helper, PropertyPath<? super Annotatable, ?> property)
	{
		super(index, id, sourceType, targetType, direction, targetRestriction, helper);
		this.property = property;
	}
	
	@Override
	public Object getValue(RelatedItemHelper helper, Annotatable item) 
	{
		Set<Annotatable> items = getRelatedItems(item);
		if (items.size() == 0) return null;

		List<Object> values = new ArrayList<>(items.size());
		for (Annotatable p : items)
		{
			Object v = property == null ? p : property.getValue(helper.dc, p);
			if (v != null) values.add(v);
		}
		
		return values.size() == 0 ? null : values;
	}

}
