/**
	$Id $

	Copyright (C) 2016 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.extensions.startpage;

import net.sf.basedb.util.extensions.Action;

/**
	Action for selecting the start page that should be displayed after 
	logging in to BASE.
	
	@author nicklas
	@since 3.8
	@see FixedStartPageFactory
*/
public interface StartPageAction
	extends Action
{

	/**
		Get the ID of the the action. This ID is stored as a user 
		setting in BASE and is used to find the correct URL after
		logging in. It is recommended that an extension use the same
		ID as the extension itself have.
	*/
	public String getId();
	
	/**
		Get the name of the start page.
	*/
	public String getName();
	
	/**
		Get an optional description of the start page.
	*/
	public String getDescription();
	
	/**
		Get the URL to the start page. The URL should be
		be an absolute URL. Use <code>$SESSION-ID$</code>
		as a placeholder for the current session id.
		Eg. <code>my_alternate_startpage.jsp?ID=$SESSION-ID$</code> 
		@return The URL to the page to load, or null to load the regular start page
	*/
	public String getUrl();
	
}
