/**
	$Id: ServletExportOutputStream.java 7154 2016-05-23 10:34:34Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.clients.web.util;

import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

import net.sf.basedb.core.plugin.ExportOutputStream;

/**
 	An extension to the {@link ExportOutputStream} class which 
 	directs the output to the response stream of a 
 	{@link HttpServletResponse} object. This allows an export plugin
 	to send data directly to the web browser without going throgh
 	the BASE filesystem or temporary files on the disk.

	@author nicklas
	@version 2.2
	@base.modified $Date: 2016-05-23 12:34:34 +0200 (Mon, 23 May 2016) $
*/
public class ServletExportOutputStream
	extends ExportOutputStream
{
	
	private final HttpServletResponse response;
	
	/**
		Create a new servlet export stream.
		@param response The servlet response object
	 	@throws IOException If getting the output stream from the reponse fails.
	*/
	public ServletExportOutputStream(HttpServletResponse response)
		throws IOException
	{
		super(response.getOutputStream());
		this.response = response;
	}
	
	/*
	 	From the ExportOutputStream class
	 	---------------------------------
	*/
	/**
		Calls {@link HttpServletResponse#setContentLength(int)}
	*/
	@Override
	public void setContentLength(long contentLength)
	{
		response.setContentLengthLong(contentLength);
	}
	/**
		Calls {@link HttpServletResponse#setContentType(String)}
	*/
	@Override
	public void setMimeType(String mimeType)
	{
		response.setContentType(mimeType);
	}
	/**
		Calls {@link HttpServletResponse#setCharacterEncoding(String)}
		@since 2.9
	*/
	@Override
	public void setCharacterSet(String charset)
	{
		response.setCharacterEncoding(charset);
	}
	/**
		Calls {@link HttpServletResponse#setHeader(java.lang.String, java.lang.String)}
		with the following values: 
		<code>response.setHeader("Content-Disposition", "attachment;filename=&lt;filename&gt;");</code>
		If the filename contains unsafe characters it is URL encoded.
	*/
	@Override
	public void setFilename(String filename)
	{
		filename = HTML.urlEncode(filename.replaceAll(" ", "."));
		response.setHeader("Content-Disposition", "attachment;filename="+filename);
	}
	// -------------------------------------------
	
}
