/**
	$Id: SimpleAbsoluteProgressReporter.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core;

/**
	This is a simple implementation of the {@link AbsoluteProgressReporter} interface,
 	which simply remembers that last values. 
 	
	@author nicklas
	@version 2.0
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (Fri, 14 Sep 2012) $
	@see SimpleProgressReporter
*/
public class SimpleAbsoluteProgressReporter
	extends SimpleProgressReporter
	implements AbsoluteProgressReporter
{

	private final long total;
	private long completed;
	private long baseCompleted;
	private String message;
	
	/**
		Create a new absolute progress reporter. If the total is set to
		-1 (unknown size), all percentages are forwarded to the parent
		as -1.
		@param progress An optional progress which is also notified about
			the progress
		@param total The end point of task, or -1 if not known
	*/
	public SimpleAbsoluteProgressReporter(ProgressReporter progress, long total)
	{
		super(progress);
		this.total = total;
	}
	
	/*
		From the AbsoluteProgressReporter interface
		-------------------------------------------
	*/	
	@Override
	public void displayAbsolute(long completed, String message)
	{
		this.completed = completed + this.baseCompleted;
		int percent = total == -1 ? -1 : (int)(100 * this.completed / total);
		display(percent, message == null ? this.message : message);
	}
	// -------------------------------------------

	/**
		Get the end point of the task.
	*/
	public long getTotal()
	{
		return total;
	}
	/**
		Get completed number of ticks of the task.
	*/
	public long getCompleted()
	{
		return completed;
	}
	
	/**
		Set a basic progress level. Subsequent calls to {@link #displayAbsolute(long, String)}
		are based on the completed value and message set here.
		@param completed The basic completed progress
		@param message A default message
		@since 2.16
	*/
	public void setAbsolute(long completed, String message)
	{
		this.baseCompleted = completed;
		this.message = message;
		displayAbsolute(0, message);
	}
}
