/*
  $Id $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Set;

/**
	This class holds information quantities.
	
	@author Nicklas
	@version 2.9
	@see net.sf.basedb.core.Quantity
	@base.modified $Date $
	@hibernate.class table="`Quantities`" lazy="true"
	@see <a href="../../../../../../html/developer/api/data_api.annotations.html">Developer documentation: Annotations</a>
*/
public class QuantityData
	extends BasicData
	implements NameableData, RemovableData, SystemData
{

	/*
		From the Nameable interface
		-------------------------------------------
	*/
	private String name;
	@Override
	public String getName()
	{
		return name;
	}
	@Override
	public void setName(String name)
	{
		this.name = name;
	}

	private String description;
	@Override
	public String getDescription()
	{
		return description;
	}

	@Override
	public void setDescription(String description)
	{
		this.description = description;
	}
	// -------------------------------------------
	/*
		From the RemovableData interface
		-------------------------------------------
	*/
	private Integer removedBy;
	@Override
	public Integer getRemovedBy()
	{
		return removedBy;
	}
	@Override
	public void setRemovedBy(Integer removedBy)
	{
		this.removedBy = removedBy;
	}
	// -------------------------------------------
	/*
		From the SystemData interface
		-------------------------------------------
	*/
	private String systemId; 
	@Override
	public String getSystemId()
	{
		return systemId;
	}
	public void setSystemId(String systemId)
	{
		this.systemId = systemId;
	}
	// -------------------------------------------

	public static final int MAX_REFERENCE_UNIT_LENGTH = 255;
	
	private String referenceUnit;
	/**
		The reference unit that all units of must relate to.
		@hibernate.property column="`reference_unit`" type="string" length="255" not-null="true"
	*/
	public String getReferenceUnit()
	{
		return referenceUnit;
	}
	public void setReferenceUnit(String referenceUnit)
	{
		this.referenceUnit = referenceUnit;
	}

	private Set<UnitData> units;
	/**
		This is the inverse end.
		@see UnitData#getQuantity()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`quantity_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.UnitData"
	*/
	Set<UnitData> getUnits()
	{
		return units;
	}
	void setUnits(Set<UnitData> units)
	{
		this.units = units;
	}
	
	private Set<UnitSymbolData> symbols;
	/**
		This is the inverse end.
		@see UnitSymbolData#getQuantity()
		@hibernate.set lazy="true" inverse="true"
		@hibernate.collection-key column="`quantity_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.UnitSymbolData"
	*/
	Set<UnitSymbolData> getSymbols()
	{
		return symbols;
	}
	void setSymbols(Set<UnitSymbolData> symbols)
	{
		this.symbols = symbols;
	}

	
}
