/**
	$Id $

	Copyright (C) 2016 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.log.db;


import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.LogControl;
import net.sf.basedb.core.data.ChangeHistoryDetailData;
import net.sf.basedb.core.data.LoggableData;
import net.sf.basedb.core.log.EntityDetails;
import net.sf.basedb.core.log.EntityLogger;
import net.sf.basedb.core.log.LogManager;
import net.sf.basedb.core.log.ManualLogEntry;

/**
	Logger for manual entries. Note that this implementation will
	ignore entries related to items not implementing the {@link LoggableData}
	interface.

	@author Nicklas
	@since 3.9
*/
public class ManualEntryLogger
	implements EntityLogger
{

	private final DbLogManagerFactory factory;
	private final boolean detailedProperties;
	private final boolean oldPropertyValues;

	/**
		Creates a new plate event logger.
		@param factory The log manager factory currently in use
	*/
	public ManualEntryLogger(DbLogManagerFactory factory)
	{
		this.factory = factory;
		this.detailedProperties = factory.isLoggingDetailedProperties();
		this.oldPropertyValues = factory.isLoggingOldPropertyValues();
	}
	
	/*
		From the EntityLogger interface
		-------------------------------
	*/
	@Override
	public void logChanges(LogManager logManager, EntityDetails details)
	{
		LogControl logControl = logManager.getLogControl();
		ManualLogEntry entry = (ManualLogEntry)details.getEntity();

		BasicItem item = entry.getItem();

		// If the item isn't implementing the LoggableData interface
		// we don't log the changes
		if (!LoggableData.class.isAssignableFrom(item.getType().getDataClass())) return;

		ChangeHistoryDetailData change = new ChangeHistoryDetailData();
		change.setChangeType(entry.getChangeType().getValue());
		change.setItemId(item.getId());
		change.setItemType(item.getType().getValue());
		change.setChangeInfo(entry.getLogInfo());
		if (oldPropertyValues)
		{
			change.setNewValue(entry.getNewValue());
			change.setOldValue(entry.getOldValue());
		}
		((DbLogManager)logManager).logChangeDetails(change, details);
	}
	// -------------------------------
}
