/*
	$Id: SqlColumnExpression.java 6127 2012-09-14 07:28:35Z nicklas $

	Copyright (C) 2006 Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.query;

import java.util.Collection;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.BaseException;
import net.sf.basedb.core.HibernateUtil;
import net.sf.basedb.core.ReporterList;
import net.sf.basedb.util.EqualsHelper;

/**
	An expression representing the column in a
	database tabe: alias.column.
	<p>
	Throws an <code>UnsupportedOperationException</code> if 
	{@link Query#getQueryType()} isn't {@link QueryType#SQL}.

	@author Nicklas
	@version 2.0
	@see Dynamic#score(ReporterList)
	@base.modified $Date: 2012-09-14 09:28:35 +0200 (Fri, 14 Sep 2012) $
*/
public class SqlColumnExpression
	implements Expression
{
	private final String alias;
	private final String column;

	SqlColumnExpression(String alias, String column)
	{
		this.alias = alias;
		this.column = column;
	}

	/*
		From the QueryElement interface
		-------------------------------------------
	*/
	@Override
	public String toQl(Query query, DbControl dc)
		throws BaseException
	{
		if (query.getQueryType() == QueryType.SQL)
		{
			return (alias == null ? query.getRootAlias() : alias) + "." + HibernateUtil.quote(column);
		}
		else
		{
			throw new UnsupportedOperationException("'sqlColumn' operation not supported by "+query.getQueryType()+" query.");
		}
	}
	/**
		@return Always null
	*/
	@Override
	public Collection<? extends QueryElement> getChildren()
	{
		return null;
	}
	// -------------------------------------------
	/*
		From the Object class
		-------------------------------------------
	*/
	@Override
	public String toString()
	{
		return (alias == null ? "" : alias + ".") + column;
	}
	/**
		This expression is equal to another SqlColumnExpression if their 
		aliases and columns are equal.
	*/
	@Override
	public boolean equals(Object other)
	{
		if (this == other) return true;
		if (other == null || this.getClass() != other.getClass()) return false;
		SqlColumnExpression o = (SqlColumnExpression)other;
		return EqualsHelper.equals(column, o.column) && EqualsHelper.equals(alias, o.alias);
	}
	@Override
	public int hashCode()
	{
		return EqualsHelper.hashCode(column, alias);
	}
	// -------------------------------------------
}
