/**
	$Id: NullNodeValidator.java 6047 2012-04-18 09:37:46Z nicklas $

	Copyright (C) 2008 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.overview.validator;

import net.sf.basedb.core.DbControl;
import net.sf.basedb.util.overview.OverviewContext;
import net.sf.basedb.util.overview.Node;

/**
	A node validator implementation that accepts all nodes,
	except missing items and denied access items (depending on configuration). 
	No validation failures are reported.

	@author Nicklas
	@version 2.10
	@base.modified $Date: 2012-04-18 11:37:46 +0200 (Wed, 18 Apr 2012) $
*/
public class NullNodeValidator<I>
	implements NodeValidator<I>
{

	private boolean acceptDenied;
	private boolean acceptMissing;
	
	/**
		Create a new validator with default settings.
	*/
	public NullNodeValidator()
	{}

	/**
		Created a new validator that accepts denied and/or missing nodes.
		@param acceptDenied TRUE to accept denied nodes, FALSE to reject
		@param acceptMissing TRUE to accept missing nodes, FALSE to reject
		@since 2.17
	*/
	public NullNodeValidator(boolean acceptDenied, boolean acceptMissing)
	{
		this.acceptDenied = acceptDenied;
		this.acceptMissing = acceptMissing;
	}
	
	/*
		From the NodeValidator interface
		--------------------------------
	*/
	/**
		@return TRUE / FALSE depening on the acceptDenied setting (default = false).
	*/
	@Override
	public boolean preDeniedItem(DbControl dc, OverviewContext context, Node parentNode)
	{
		return acceptDenied;
	}

	/**
		@return TRUE / FALSE depening on the acceptMissing setting (default = false).
	*/
	@Override
	public boolean preMissingItem(DbControl dc, OverviewContext context, Node parentNode)
	{
		return acceptMissing;
	}

	/**
		@return Always true
	*/
	@Override
	public boolean preValidate(DbControl dc, OverviewContext context, I item, Node parentNode)
	{
		return true;
	}
	
	/**
		Do nothing.
	*/
	@Override
	public void postMissingItem(DbControl dc, OverviewContext context, Node node, Node parentNode)
	{}
	
	/**
		Do nothing.
	*/
	@Override
	public void postDeniedItem(DbControl dc, OverviewContext context, Node node, Node parentNode)
	{}
	
	/**
		Do nothing.
	*/
	@Override
	public void postValidate(DbControl dc, OverviewContext context, Node node, Node parentNode)
	{}
	
	/**
		Do nothing.
	*/
	@Override
	public void postValidateFolder(DbControl dc, OverviewContext context, Node folderNode, Node parentNode)
	{}
	// -------------------------------------

}
