<%-- $Id: list_users.jsp 7914 2021-02-23 07:23:26Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg, Martin Svensson
	Copyright (C) 2007 Johan Enell, Nicklas Nordborg, Martin Svensson

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.ExtendedProperty"
	import="net.sf.basedb.core.ExtendedProperties"	
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.User"
	import="net.sf.basedb.core.Group"
	import="net.sf.basedb.core.Role"
	import="net.sf.basedb.core.Type"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.Include"
	import="net.sf.basedb.core.ItemResultIterator"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.PluginDefinition"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.core.plugin.Plugin"
	import="net.sf.basedb.util.Enumeration"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.ModeInfo"
	import="net.sf.basedb.clients.web.PermissionUtil"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction" 
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnAction"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnUtil"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="java.util.List"
	import="java.util.Map"
	import="java.util.Date"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%!
	private static final Item itemType = Item.USER;
	private static final GuiContext guiContext = new GuiContext(itemType, GuiContext.Type.LIST);
%>
<%
final SessionControl sc = Base.getExistingSessionControl(pageContext, Permission.DENIED, itemType);
final String ID = sc.getId();
final boolean createPermission = sc.hasPermission(Permission.CREATE, itemType);
final boolean writePermission = sc.hasPermission(Permission.WRITE, itemType);
final boolean deletePermission = sc.hasPermission(Permission.DELETE, itemType);
final boolean impersonatePermission = sc.hasSystemPermission(Permission.ACT_AS_ANOTHER_USER);
final ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, null, null);

final ModeInfo mode = ModeInfo.get(request.getParameter("mode"));
final String callback = request.getParameter("callback");
final String title = mode.generateTitle("user", "users");
final DbControl dc = sc.newDbControl();
ItemResultIterator<User> users = null;
try
{
	Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);

	// Query for groups relatated to the current user
	final ItemQuery<Group> groupQuery = Group.getQuery();
	groupQuery.include(cc.getInclude());
	groupQuery.join(Hql.innerJoin("users", "usr"));
	groupQuery.restrict(Restrictions.eq(Hql.alias("usr"), Expressions.parameter("user")));
	groupQuery.order(Orders.asc(Hql.property("name"))); 

	// Query for roles relatated to the current user
	final ItemQuery<Role> roleQuery = Role.getQuery();
	roleQuery.include(cc.getInclude());
	roleQuery.join(Hql.innerJoin("users", "usr"));
	roleQuery.restrict(Restrictions.eq(Hql.alias("usr"), Expressions.parameter("user")));
	roleQuery.order(Orders.asc(Hql.property("name"))); 
	
	List<ExtendedProperty> extendedProperties = ExtendedProperties.getProperties("UserData");
	
	Map<Plugin.MainType, Integer> pluginCount = PluginDefinition.countPlugins(dc, guiContext);
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, guiContext, null);
	try
	{
		final ItemQuery<User> query = Base.getConfiguredQuery(dc, cc, jspContext, true, User.getQuery(), mode);
		users = query.iterate(dc);
	}
	catch (Throwable t)
	{
		cc.setMessage(t.getMessage());
	}
	int numListed = 0;
	ExtensionsInvoker<ButtonAction> invoker = ToolbarUtil.useExtensions(jspContext);
	ExtensionsInvoker<ListColumnAction<User,?>> columnsInvoker = ListColumnUtil.useExtensions(jspContext);
	%>
	<base:page title="<%=title==null ? "Users" : title%>" type="<%=mode.getPageType()%>" id="list-page">
	<base:head scripts="table.js,~users.js" styles="table.css,toolbar.css">
		<ext:scripts context="<%=jspContext%>" />
		<ext:stylesheets context="<%=jspContext%>" />
	</base:head>
	
	<base:body>
		<h1><%=title==null ? "Users" : title%></h1>
		<div class="content">
		<tbl:table 
			id="users" 
			columns="<%=cc.getSetting("columns")%>"
			sortby="<%=cc.getSortProperty()%>" 
			direction="<%=cc.getSortDirection()%>"
			action="index.jsp"
			sc="<%=sc%>"
			item="<%=itemType%>"
			filterrows="<%=cc.getFilterRows()%>"
			subclass="fulltable"
			>
			<tbl:hidden 
				name="mode" 
				value="<%=mode.getName()%>" 
			/>
			<tbl:hidden 
				name="callback" 
				value="<%=callback%>" 
				skip="<%=callback == null%>" 
			/>
			<tbl:columndef 
				id="name"
				property="name"
				datatype="string"
				title="Name"
				sortable="true" 
				filterable="true"
				exportable="true"
				show="always" 
			/>
			<tbl:columndef 
				id="id"
				clazz="uniquecol"
				property="id"
				datatype="int"
				title="ID"
				sortable="true"
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="entryDate"
				property="entryDate"
				datatype="date"
				title="Registered"
				sortable="true" 
				filterable="true"
				exportable="true"
				formatter="<%=dateFormatter%>"
			/>
			<tbl:columndef 
				id="login"
				clazz="uniquecol"
				property="login"
				datatype="string"
				title="Login"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="systemId"
				clazz="uniquecol"
				property="systemId"
				datatype="string"
				title="System ID"
				sortable="true"
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="externalId"
				clazz="uniquecol"
				property="externalId"
				datatype="string"
				title="External ID"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="expirationDate"
				property="expirationDate"
				datatype="date"
				title="Expiration date"
				sortable="true" 
				filterable="true"
				exportable="true"
				formatter="<%=dateFormatter%>"
			/>
			<tbl:columndef 
				id="disabled"
				property="disabled"
				datatype="boolean"
				title="Disabled"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="multiuserAccount"
				property="multiuserAccount"
				datatype="boolean"
				title="Multi-user account"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="useDeviceVerification"
				property="useDeviceVerification"
				datatype="boolean"
				title="2-factor login"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="groups"
				title="Groups"
				property="&groups(name)"
				datatype="string"
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef
				id="roles"
				title="Roles"
				property="&roles(name)"
				datatype="string"
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="organisation"
				property="organisation"
				datatype="string"
				title="Organisation"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="address"
				property="address"
				datatype="string"
				title="Address"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="email"
				property="email"
				datatype="string"
				title="Email"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="phone"
				property="phone"
				datatype="string"
				title="Phone"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="fax"
				property="fax"
				datatype="string"
				title="Fax"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="url"
				property="url"
				datatype="string"
				title="Url"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<%
			if (extendedProperties != null)
			{
				for (ExtendedProperty ep : extendedProperties)
				{
					String name = ep.getName();
					%>
					<tbl:columndef
						id="<%=name%>"
						property="<%=name%>"
						datatype="<%=ep.getType().getStringValue()%>"
						title="<%=HTML.encodeTags(ep.getTitle())%>"
						sortable="true"
						filterable="true"
						exportable="true"
						formatter="<%=FormatterFactory.getExtendedPropertyFormatter(sc, ep)%>"
					/>
					<%
				}
			}
			%>
			<tbl:columndef 
				id="quota"
				property="quota.name"
				datatype="string"
				title="Quota"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="quotagroup"
				property="quotaGroup.name"
				datatype="string"
				title="Quota group"
				sortable="true" 
				filterable="true"
				exportable="true"
			/>
			<tbl:columndef 
				id="description"
				property="description"
				datatype="string"
				title="Description" 
				sortable="true" 
				filterable="true" 
				exportable="true"
			/>
			<tbl:columndef 
				id="xt-columns" 
				extensions="<%=columnsInvoker%>" 
				jspcontext="<%=jspContext%>" 
			/>
			<div class="panelgroup bg-filled-50 bottomborder">
				<tbl:toolbar
					subclass="bottomborder"
					visible="<%=mode.hasToolbar()%>"
					>
					<tbl:button 
						id="btnNewItem"
						disabled="<%=!createPermission%>" 
						image="new.png" 
						title="New&hellip;" 
						tooltip="<%=createPermission ? "Create new user" : "You do not have permission to create users"%>" 
					/>
					<tbl:button 
						id="btnDeleteItems"
						disabled="<%=!deletePermission%>" 
						image="delete.png" 
						title="Delete" 
						tooltip="<%=deletePermission ? "Delete the selected items" : "You do not have permission to delete users" %>" 
					/>
					<tbl:button 
						id="btnRestoreItems"
						disabled="<%=!writePermission%>" 
						image="restore.png" 
						title="Restore" 
						tooltip="<%=writePermission ? "Restore the selected (deleted) items" : "You do not have permission to edit users" %>" 
					/>
					<tbl:button 
						id="btnColumns"
						image="columns.png" 
						title="Columns&hellip;" 
						tooltip="Show, hide and re-order columns" 
					/>
					<tbl:button
						id="btnDefaultMembership"
						image="membership.png"
						title="Default membership&hellip;"
						tooltip="Specify groups and roles that are automtically assigned to new users"
					/>
					<tbl:button
						id="btnNewGroup"
						image="add.png"
						title="New group&hellip;"
						tooltip="Create a new group with the selected users as members"
						visible="<%=sc.hasPermission(Permission.CREATE, Item.GROUP)%>"
					/>
					<tbl:button 
						id="btnImport"
						data-plugin-type="IMPORT"
						image="import.png" 
						title="Import&hellip;" 
						tooltip="Import data" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.IMPORT)%>"
					/>
					<tbl:button 
						id="btnExport"
						data-plugin-type="EXPORT"
						image="export.png" 
						title="Export&hellip;" 
						tooltip="Export data" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.EXPORT)%>"
					/>
					<tbl:button 
						id="btnRunPlugin"
						data-plugin-type="OTHER"
						image="runplugin.png" 
						title="Run plugin&hellip;" 
						tooltip="Run a plugin" 
						visible="<%=pluginCount.containsKey(Plugin.MainType.OTHER)%>"
					/>
					<ext:render extensions="<%=invoker%>" context="<%=jspContext%>" 
						wrapper="<%=new PrefixSuffixRenderer<ButtonAction>(jspContext, "<td>", "</td>") %>"/>
				</tbl:toolbar>
				<tbl:panel>
					<tbl:presetselector />
					<tbl:navigator
						page="<%=cc.getPage()%>" 
						rowsperpage="<%=cc.getRowsPerPage()%>" 
						totalrows="<%=users == null ? 0 : users.getTotalCount()%>" 
						visible="<%=mode.hasNavigator()%>"
					/>
				</tbl:panel>
			</div>
			<tbl:data>
				<tbl:headers>
					<tbl:headerrow>
						<tbl:header colspan="3" />
						<tbl:columnheaders />
					</tbl:headerrow>
					<%
					int numFilters = cc.getNumPropertyFilters();
					int numRows = cc.getFilterRows();
					for (int filterNo = 0; filterNo < numRows; filterNo++)
					{
						boolean lastRow = filterNo == numRows-1;
						%>
						<tbl:headerrow>
							<tbl:header subclass="index" />
							<tbl:header 
								subclass="check" 
								visible="<%=mode.hasCheck()%>"
								><base:icon 
									subclass="link table-check"
									image="check_uncheck.png" 
									tooltip="Toggle all (use CTRL, ALT or SHIFT to check/uncheck)" 
									visible="<%=lastRow%>"
								/></tbl:header>
							<tbl:header 
								subclass="check" 
								visible="<%=mode.hasRadio()%>"
								/>
							<tbl:header 
								subclass="icons" 
								visible="<%=mode.hasIcons()%>"
								>
								<base:icon
									subclass="link table-filter-row-action"
									image="add.png"
									tooltip="Add extra filter row"
									visible="<%=lastRow%>"
								/><base:icon
									subclass="link table-filter-row-action"
									image="remove.png"
									tooltip="Remove this filter row"
									visible="<%=numRows > 1 || numFilters > 0 %>"
									data-remove-row="<%=filterNo%>"
								/>
							</tbl:header>
							<tbl:propertyfilter row="<%=filterNo%>" />
						</tbl:headerrow>
						<%
					}
					%>
					<tbl:columnsubtitles />
				</tbl:headers>
				<tbl:rows>
					<%
					if (cc.getMessage() != null)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer error"><%=cc.getMessage()%></div>
						</tbl:panel>
						<%
						cc.setMessage(null);
					}
					int index = cc.getPage()*cc.getRowsPerPage();
					int selectedItemId = cc.getId();
					if (users != null)
					{						
						while (users.hasNext())
						{
							User item = users.next();
							int itemId = item.getId();
							String name = HTML.encodeTags(item.getName());
							String tooltip = mode.isSelectionMode() ? 
									"Select this item" : "View this item" + (writePermission ? " (use CTRL, ALT or SHIFT to edit)" : "");
							
							index++;
							numListed++;
							%>
							<tbl:row>
								<tbl:header 
									clazz="index"
									><%=index%></tbl:header>
								<tbl:header 
									clazz="check" 
									visible="<%=mode.hasCheck()%>"
									><input 
										type="checkbox" 
										name="<%=itemId%>" 
										value="<%=itemId%>" 
										title="<%=name%>" 
										<%=cc.getSelected().contains(itemId) ? "checked" : ""%> 
									></tbl:header>
								<tbl:header 
									clazz="check" 
									visible="<%=mode.hasRadio()%>"
									><input 
										type="radio" 
										name="item_id" 
										value="<%=itemId%>" 
										title="<%=name%>" 
										<%=selectedItemId == itemId ? "checked" : ""%>
									></tbl:header>
								<tbl:header 
									clazz="icons" 
									visible="<%=mode.hasIcons()%>"
									><base:icon 
										image="deleted.png"
										id="<%="delete."+itemId %>"
										subclass="<%=deletePermission ? "table-delete-item" : "disabled" %>"
										data-item-id="<%=itemId%>"
										tooltip="This item has been scheduled for deletion" 
										visible="<%=item.isRemoved()%>"
									/><base:icon
										image="systemitem.png"
										tooltip="This item is a system item"
										visible="<%=item.isSystemItem()%>" 
									/><base:icon
										image="login.png"
										id="<%="impersonate."+itemId %>"
										tooltip="Login as this user"
										subclass="auto-init"
										data-auto-init="impersonate"
										data-item-id="<%=itemId %>"
										visible="<%=impersonatePermission%>"
									/><base:icon
										image="copy.png"
										id="<%="clone."+itemId %>"
										tooltip="Create a new user using settings from this user as a template"
										subclass="auto-init"
										data-auto-init="clone"
										data-item-id="<%=itemId %>"
										visible="<%=createPermission%>"
									/>&nbsp;</tbl:header>
								<tbl:cell column="name"><div 
									class="link table-item"
									data-item-id="<%=itemId%>"
									data-no-edit="<%=writePermission ? 0 : 1 %>" 
									tabindex="0"
									title="<%=tooltip%>"><%=name%></div></tbl:cell>
								<tbl:cell column="id"><%=item.getId()%></tbl:cell>
								<tbl:cell column="entryDate" value="<%=item.getEntryDate()%>" />
								<tbl:cell column="login"><%=HTML.encodeTags(item.getLogin())%></tbl:cell>
								<tbl:cell column="systemId"><%=Values.getString(item.getSystemId())%></tbl:cell>
								<tbl:cell column="externalId"><%=HTML.encodeTags(item.getExternalId())%></tbl:cell>
								<tbl:cell column="expirationDate" value="<%=item.getExpirationDate()%>" />
								<tbl:cell column="disabled"><%=item.isDisabled() ? "yes" : "no" %></tbl:cell>
								<tbl:cell column="multiuserAccount"><%=item.isMultiuserAccount() ? "yes" : "no" %></tbl:cell>
								<tbl:cell column="useDeviceVerification"><%=item.getUseDeviceVerification() ? "yes" : "no" %></tbl:cell>
								<tbl:cell column="groups">
									<%
									groupQuery.setParameter("user", itemId, Type.INT);
									try
									{
										String separator = "";
										for (Group g : groupQuery.list(dc))
										{
											out.write(separator);
											if (mode.hasPropertyLink())
											{
												out.write(Base.getLinkedName(ID, g, false, mode.hasEditLink()));
											}
											else
											{
												out.write(HTML.encodeTags(g.getName()));
											}
											separator = ", ";
										}
									}
									catch (Throwable t)
									{
										%>
										<div class="error"><%=t.getMessage()%></div>
										<%
									}
									%>
								</tbl:cell>
								<tbl:cell column="roles">
									<%
									roleQuery.setParameter("user", itemId, Type.INT);
									try
									{
										String separator = "";
										for (Role r : roleQuery.list(dc))
										{
											out.write(separator);
											if (mode.hasPropertyLink())
											{
												out.write(Base.getLinkedName(ID, r, false, mode.hasEditLink()));
											}
											else
											{
												out.write(HTML.encodeTags(r.getName()));
											}
											separator = ", ";
										}
									}
									catch (Throwable t)
									{
										%>
										<div class="error"><%=t.getMessage()%></div>
										<%
									}
									%>
								</tbl:cell>
								<tbl:cell column="organisation"><%=HTML.encodeTags(item.getOrganisation())%></tbl:cell>
								<tbl:cell column="address"><%=HTML.encodeTags(item.getAddress())%></tbl:cell>
								<tbl:cell column="email"><%=HTML.encodeTags(item.getEmail())%></tbl:cell>
								<tbl:cell column="phone"><%=HTML.encodeTags(item.getPhone())%></tbl:cell>
								<tbl:cell column="fax"><%=HTML.encodeTags(item.getFax())%></tbl:cell>
								<tbl:cell column="url"><%=HTML.encodeTags(item.getUrl())%></tbl:cell><%
								if (extendedProperties != null)
								{
									for (ExtendedProperty ep : extendedProperties)
									{
										name = ep.getName();
										%>
										<tbl:cell column="<%=name%>"><tbl:cellvalue value="<%=item.getExtended(name)%>" /></tbl:cell>
										<%
									}
								}
								%>
								<tbl:cell column="quota"
									><base:propertyvalue 
										item="<%=item%>" 
										property="quota"
										enableEditLink="<%=mode.hasEditLink()%>" 
										enablePropertyLink="<%=mode.hasPropertyLink()%>"
									/></tbl:cell>
								<tbl:cell column="quotagroup"
									><base:propertyvalue 
										item="<%=item%>" 
										property="quotaGroup"
										enableEditLink="<%=mode.hasEditLink()%>" 
										enablePropertyLink="<%=mode.hasPropertyLink()%>"
									/></tbl:cell>
								<tbl:cell column="description"><%=HTML.encodeTags(item.getDescription())%></tbl:cell>
								<tbl:xt-cells dc="<%=dc%>" item="<%=item%>">
									<tbl:cell column="xt-columns" />
								</tbl:xt-cells>
							</tbl:row>
							<%
							}
						}
					if (numListed == 0)
					{
						%>
						<tbl:panel subclass="bg-filled-50">
							<div class="messagecontainer note">
							<%=users == null || users.getTotalCount() == 0 ? "No users were found" : "No users on this page. Please select another page!" %>
							</div>
						</tbl:panel>
						<%
					}
					%>
				</tbl:rows>
			</tbl:data>
		</tbl:table>
		</div>
		
		<base:buttongroup subclass="dialogbuttons">
			<base:button id="btnOk" title="Ok" visible="<%=mode.hasOkButton()%>" />
			<base:button id="close" title="Cancel" visible="<%=mode.hasCancelButton()%>" />
			<base:button id="close" title="Close" visible="<%=mode.hasCloseButton()%>" />
		</base:buttongroup>
		
	</base:body>
	</base:page>
	<%
}
finally
{
	if (users != null) users.close();
	if (dc != null) dc.close();
}
%>
