<%-- $Id: list_anytoany.jsp 7913 2021-02-22 07:16:45Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.Permission"
	import="net.sf.basedb.core.BasicItem"
	import="net.sf.basedb.core.AnyToAny"
	import="net.sf.basedb.core.Nameable"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.Subtypable"
	import="net.sf.basedb.core.Removable"
	import="net.sf.basedb.core.File"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.PermissionDeniedException"
	import="net.sf.basedb.core.ItemNotFoundException"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.extensions.ExtensionsInvoker"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="net.sf.basedb.clients.web.extensions.ExtensionsControl"
	import="net.sf.basedb.clients.web.extensions.JspContext"
	import="net.sf.basedb.clients.web.extensions.toolbar.ButtonAction" 
	import="net.sf.basedb.clients.web.extensions.toolbar.ToolbarUtil"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnAction"
	import="net.sf.basedb.clients.web.extensions.list.ListColumnUtil"
	import="net.sf.basedb.clients.web.extensions.fileviewer.FileViewerContext"
	import="net.sf.basedb.clients.web.extensions.fileviewer.FileViewerUtil"
	import="net.sf.basedb.clients.web.extensions.renderer.PrefixSuffixRenderer"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
String root = request.getContextPath() + "/";
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final Item itemType = Item.valueOf(request.getParameter("item_type"));
final int itemId = Values.getInt(request.getParameter("item_id"));
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
final ItemContext cc = sc.getCurrentContext(itemType);
String title = Values.getString(request.getParameter("title"), "Related items");

try
{
	final BasicItem item = itemType.getById(dc, itemId);
	final boolean createPermission = item.hasPermission(Permission.WRITE);
	final boolean deletePermission = createPermission;
	
	ItemQuery<AnyToAny> query = AnyToAny.getLinksFrom(item);
	query.order(Orders.asc(Hql.property("name")));
	ItemResultList<AnyToAny> links = query.list(dc);
	long count = links.getTotalCount();
	
	JspContext jspContext = ExtensionsControl.createContext(dc, pageContext, new GuiContext(Item.ANYTOANY, GuiContext.Type.LIST), item);
	ExtensionsInvoker<ButtonAction> invoker = ToolbarUtil.useExtensions(jspContext);
	ExtensionsInvoker<ListColumnAction<AnyToAny,?>> columnsInvoker = ListColumnUtil.useExtensions(jspContext);
	FileViewerContext fileContext = FileViewerContext.anyToAny(item);
	FileViewerUtil fileViewer = FileViewerUtil.getOrCreate(jspContext, fileContext);
	%>
	<base:page type="include">
	<base:head scripts="/common/anytoany/list.js" />
	<base:body>
		<ext:scripts context="<%=jspContext%>" />
		<base:section 
			id="anytoany" 
			title="<%=title + " (" + (count) +")"%>"
			context="<%=cc%>">

				<tbl:table
					id="links"
					columns="all"
					sc="<%=sc%>"
					action="<%=root + "common/anytoany/index.jsp"%>"
					>
					<tbl:columndef 
						id="name"
						title="Name"
					/>
					<tbl:columndef 
						id="item"
						title="Item"
					/>
					<tbl:columndef 
						id="description"
						title="Description"
					/>
					<tbl:columndef 
						id="xt-columns" 
						extensions="<%=columnsInvoker%>" 
						jspcontext="<%=jspContext%>" 
					/>
					<tbl:toolbar
						subclass="topborder leftborder rightborder bg-filled-50"
						>
						<tbl:button 
							id="btnNewLink"
							disabled="<%=!createPermission%>" 
							image="add.png" 
							title="Add link&hellip;" 
							tooltip="<%=createPermission ? "Create new link" : "You do not have permission to create links"%>" 
							data-item-type="ANYTOANY"
							data-extra-url="<%="&from_type="+itemType.name() + "&from_id="+itemId %>"
						/>
						<tbl:button 
							id="btnUnlinkItems"
							image="delete_permanently.png"
							title="Unlink&hellip;" 
							tooltip="Unlink the selected items"
							visible="<%=count > 0 %>"
							disabled="<%=!deletePermission%>"
						/>
						<ext:render extensions="<%=invoker%>" context="<%=jspContext%>" 
							wrapper="<%=new PrefixSuffixRenderer<ButtonAction>(jspContext, "<td>", "</td>") %>"/>
					</tbl:toolbar>
					<tbl:data>
						<tbl:headers style="<%=links.size() == 0 ? "display: none;" : "" %>">
							<tbl:headerrow>
							<tbl:header subclass="index" />
							<tbl:header 
								subclass="check"
								><base:icon 
									subclass="link table-check"
									image="check_uncheck.png" 
									tooltip="Toggle all (use CTRL, ALT or SHIFT to check/uncheck)" 
								/></tbl:header>
								<tbl:columnheaders />
							</tbl:headerrow>
							<tbl:columnsubtitles />
						</tbl:headers>
						<tbl:rows>
						<%
						int index = 0;
						for (AnyToAny ata : links)
						{
							index++;
							String name = HTML.encodeTags(ata.getName());
							int ataId = ata.getId();
							BasicItem to = null;
							String subtype = ata.getToType().toString();
							boolean readTo = true;
							boolean missingTo = false;
							try
							{
								to = ata.getTo();
							}
							catch (PermissionDeniedException ex)
							{
								readTo = false;
							}
							catch (ItemNotFoundException ex)
							{
								missingTo = true;
							}
							
							String toLink = "";
							if (to instanceof Nameable)
							{
								Nameable nameable = (Nameable)to;
								toLink = Base.getLinkedName(ID, nameable, false, true);
							}
							else if (!readTo)
							{
								toLink = "<i>- denied -</i>";
							}
							else if (missingTo)
							{
								toLink = "<i>- missing -</i> (" + ata.getToType().toString() + "; id=" + ata.getToId() + ")";
							}
							else if (to == null)
							{
								toLink = "<i>- none -</i>";
							}
							else
							{
								toLink = Base.getLink(ID, to.toString(), to.getType(), to.getId(), true);
							}
							if (to instanceof File)
							{
								File toFile = (File)to;
								fileContext.setCurrentAnyToAny(toFile, ata);
								toLink += " " + fileViewer.renderAsString();
							}
							if (to instanceof Subtypable)
							{
								try
								{
									ItemSubtype st = ((Subtypable)to).getItemSubtype();
									if (st != null) subtype = st.getName();
								}
								catch (RuntimeException ex)
								{}
							}
							boolean isRemoved = false;
							if (to instanceof Removable)
							{
								isRemoved = ((Removable)to).isRemoved();
							}
							toLink += " <span class=\"itemsubtype\">("+HTML.encodeTags(subtype)+")</span>";
							%>
							<tbl:row>
								<tbl:header 
									clazz="index"
									><%=index%></tbl:header>
								<tbl:header 
									clazz="check" 
									><input 
										type="checkbox" 
										name="<%=ataId%>" 
										value="<%=ataId%>" 
										title="<%=name%>"
									></tbl:header>
								<tbl:cell column="name"><%=Base.getLinkedName(ID, ata, false, true)%></tbl:cell>
								<tbl:cell column="item"><base:icon image="deleted.png"
									tooltip="This item has been scheduled for deletion"
									visible="<%=isRemoved%>" />
									<%=toLink%></tbl:cell>
								<tbl:cell column="description"><%=HTML.niceFormat(ata.getDescription())%></tbl:cell>
								<tbl:xt-cells dc="<%=dc%>" item="<%=ata%>">
									<tbl:cell column="xt-columns" />
								</tbl:xt-cells>
							</tbl:row>
							<%
						}
						if (count == 0)
						{
							%>
							<tbl:panel>
								There are no other items related to this
								<%=itemType.toString().toLowerCase() %>.
							</tbl:panel>
							<%
						}
						%>
						</tbl:rows>
					</tbl:data>
				</tbl:table>
		</base:section>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}

%>
