/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Configure = function()
{
	var columnDefs;
	var currentColumns;
	
	var configure = {};
	
	/**
		Initialize the page.
	*/
	configure.initPage = function()
	{
		var tableId = Data.get('page-data', 'table-id');
		
		// Load default and currently visible columns
		columnDefs = window.opener.Table.getColumnDefs(tableId);
		currentColumns = window.opener.Table.getColumns(tableId);
		
		// Initialize the visible and hidden columns lists
		configure.initColumns(currentColumns);
		
		// Buttons
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnSave', configure.save);
		Buttons.addClickHandler('btnSavePreset', configure.savePresetAs);
		Events.addEventHandler('presets', 'change', configure.presetOnChange);
		if (Doc.element('selectAnnotationTypes'))
		{
			Buttons.addClickHandler('selectAnnotationTypes', configure.selectAnnotationTypes);
			Events.addEventHandler('selectAnnotationTypes', 'base-selected', configure.annotationTypeSelected);
		}
		if (Doc.element('selectRelatedItemColumn'))
		{
			Buttons.addClickHandler('selectRelatedItemColumn', configure.selectRelatedItemColumn);
			Events.addEventHandler('selectRelatedItemColumn', 'base-selected', configure.relatedItemColumnSelected);
		}
		if (Doc.element('selectLinkedItemColumn'))
		{
			Buttons.addClickHandler('selectLinkedItemColumn', configure.selectLinkedItemColumn);
			Events.addEventHandler('selectLinkedItemColumn', 'base-selected', configure.linkedItemColumnSelected);
		}
		
		Buttons.addClickHandler('moveUp', configure.moveVisibleUpOrDown);
		Buttons.addClickHandler('moveDown', configure.moveVisibleUpOrDown);
		Buttons.addClickHandler('moveLeft', configure.moveToVisible);
		Buttons.addClickHandler('moveRight', configure.moveToHidden);
		Events.addEventHandler('visible', 'dblclick', configure.moveToHidden);
		Events.addEventHandler('hidden', 'dblclick', configure.moveToVisible);
	}
	
	/**
		Initialize the visible and hidden columns lists.
		@param columns A comma-separated string with visible columns
	*/
	configure.initColumns = function(columns)
	{
		var frm = document.forms['columns'];
		// Remove all entries from the 'visible' and 'hidden' lists
		var visible = frm.visible;
		var hidden = frm.hidden;
		visible.length = 0;
		hidden.length = 0;
		
		// The visibled columns (comma-separated)
		var allVisible = columns == 'all';
		var visibleCols = columns.split(',');

		// Put all column in the hidden or visible list
		var toList = allVisible ? visible : hidden;
		for (var i = 0; i < columnDefs.length; i++) 
		{
			var col = columnDefs[i];
			if (!col.alwaysHide) configure.addColumn(toList, col);
		}
		
		// Move visible columns to the visible list
		if (!allVisible)
		{
			for (var i = 0; i < visibleCols.length; i++)
			{
				var colId = visibleCols[i];
				var col = columnDefs['id'+colId];
				for (var j = 0; j < hidden.length; j++)
				{
					if (hidden[j].value == colId)
					{
						configure.addColumn(visible, col);
						hidden[j] = null;
						j = hidden.length;
					}
				}
			}
			
			// Move columns that must be visible to the visible list
			for (var i = 0; i < hidden.length; i++)
			{
				var col = columnDefs['id'+hidden.options[i].value];
				if (col && col.alwaysShow)
				{
					configure.addColumn(visible, col);
					hidden[i] = null; 
					i--;
				}
			}
		}

	}
	
	/**
		Add a column to a list control.
	*/
	configure.addColumn = function(list, column)
	{
		var option = new Option(column.title, column.id);
		option.title = column.title;
		if (column.alwaysShow)
		{
			option.style.fontWeight = 'bold';
			option.text = option.text+' ×';
		}
		list[list.length] = option;
	}
	
	/**
		Event handler that updates the visible and hidden columns lists
		with the seleted preset.
	*/
	configure.presetOnChange = function()
	{
		var frm = document.forms['columns'];
		var presets = frm.presets;
		var columns = presets.value;
		if (columns == '') return;
		if (columns == '_default_') columns = Data.get('page-data', 'default-columns');
		if (columns == '_current_') columns = currentColumns;
		if (columns == '_minimal_')
		{
			var temp = [];
			for (var i = 0; i < columnDefs.length; i++)
			{
				var col = columnDefs[i];
				if (col.alwaysShow) temp[temp.length] = col.id;
			}
			columns = temp.join(',');
		}
		configure.initColumns(columns);
		presets.selectedIndex = 0;
	}
	
	/**
		Open a popup dialog that saves the current visible columns as a preset.
	 */
	configure.savePresetAs = function()
	{
		var visible = configure.getVisibleColumns();
		var url = 'save_preset.jsp?ID='+App.getSessionId();
		url += '&item_type='+Data.get('page-data', 'item-type');
		url += '&subcontext='+Data.get('page-data', 'subcontext', '');
		url += '&settingName='+Data.get('page-data', 'setting-name');
		url += '&columns='+visible;
		Dialogs.openPopup(url, 'SavePreset', 450, 300);
	}

	
	configure.selectAnnotationTypes = function(event)
	{
		var frm = document.forms['columns'];
		var exclude = [];
		for (var i = 0; i < frm.visible.length; i++)
		{
			if (frm.visible[i].value.indexOf('ia')==0)
			{
				exclude[exclude.length] = frm.visible[i].value.substring(2);
			}
		}
		var url = '&exclude='+exclude.join(',');
		url += '&resetTemporary=1&tmpfilter:BOOLEAN:disableInheritance=false';
		Dialogs.selectItem('ANNOTATIONTYPE', event.currentTarget.id, 1, url);
	}
	
	configure.annotationTypeSelected = function(event)
	{
		var frm = document.forms['columns'];
		var item = event.detail;
		var colId = 'ia'+item.id;
		// Do not add duplicates
		for (var i = 0; i < frm.visible.length; i++)
		{
			if (frm.visible[i].value == colId) return;
		}
		// Remove from 'hidden' list
		for (var i = frm.hidden.length-1; i >= 0; i--)
		{
			if (frm.hidden[i].value == colId) frm.hidden[i] = null;
		}
		frm.visible[frm.visible.length] = new Option(item.name + ' [I]', colId);
	}
	
	
	configure.selectRelatedItemColumn = function(event)
	{
		var frm = document.forms['columns'];
		var url = 'add_relateditem_column.jsp?ID='+App.getSessionId();
		url += '&item_type='+Data.get('page-data', 'item-type');
		Dialogs.openPopup(url, 'SelectRelatedItemColumn', 600, 400);
	}

	configure.relatedItemColumnSelected = function(event)
	{
		var frm = document.forms['columns'];
		var data = event.detail;
		// Do not add duplicates
		for (var i = 0; i < frm.visible.length; i++)
		{
			if (frm.visible[i].value == data.property) return;
		}
		frm.visible[frm.visible.length] = new Option(data.title, data.property);
	}
	
	configure.selectLinkedItemColumn = function(event)
	{
		var frm = document.forms['columns'];
		var url = 'add_linkeditem_column.jsp?ID='+App.getSessionId();
		url += '&item_type='+Data.get('page-data', 'item-type');
		url += '&subcontext='+encodeURIComponent(Data.get('page-data', 'subcontext') || '');
		Dialogs.openPopup(url, 'SelectLinkedItemColumn', 600, 400);
	}

	configure.linkedItemColumnSelected = function(event)
	{
		var frm = document.forms['columns'];
		var data = event.detail;
		// Do not add duplicates
		for (var i = 0; i < frm.visible.length; i++)
		{
			if (frm.visible[i].value == data.property) return;
		}
		frm.visible[frm.visible.length] = new Option(data.title, data.property);
	}
	
	/**
		Save the current selection and close the dialog.
	*/
	configure.save = function()
	{
		var tableId = Data.get('page-data', 'table-id');
		var visible = configure.getVisibleColumns();
		window.opener.Table.setColumns(tableId, visible);
		window.close();
	}
	
	
	// Get all visible columns as a comma-separated string
	configure.getVisibleColumns = function()
	{
		var list = document.forms['columns'].visible;
		var visible = [];
		for (var i = 0; i < list.options.length; i++)
		{
			visible[visible.length] = list.options[i].value;
		}
		return visible.join(',');
	}

	/**
		Move the selected options in the 'visible' list up or down.
	*/
	configure.moveVisibleUpOrDown = function(event)
	{
		var moveDown = Data.int(event.currentTarget, 'down');
		Forms.moveListOptions('visible', moveDown);
	}
	
	/**
		Move the selected options in the 'hidden' list to the 'visible' list.
	*/
	configure.moveToVisible = function()
	{
		var frm = document.forms['columns'];
		Forms.moveSelected(frm.hidden, frm.visible);
	}
	
	/**
		Move the selected options in the 'visible' list to the 'hidden' list.
	*/
	configure.moveToHidden = function()
	{
		var frm = document.forms['columns'];
		Forms.moveSelected(frm.visible, frm.hidden);
	}

	
	return configure;
}();

Doc.onLoad(Configure.initPage);