<%-- $Id $
	------------------------------------------------------------------
	Copyright (C) 2020 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.ItemResultList"
	import="net.sf.basedb.core.ItemList"
	import="net.sf.basedb.core.RawDataType"
	import="net.sf.basedb.core.query.Orders"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.core.query.Expressions"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.plugin.GuiContext"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="java.util.Date"
	import="java.util.List"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%@ taglib prefix="ext" uri="/WEB-INF/extensions.tld" %>
<%
String root = request.getContextPath() + "/";
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final Item itemType = Item.valueOf(request.getParameter("item_type"));
final int itemId = Values.getInt(request.getParameter("item_id"));
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
final ItemContext cc = sc.getCurrentContext(itemType);
String title = Values.getString(request.getParameter("title"), "Related items");

try
{
	Formatter<Date> timeFormatter = FormatterFactory.getDateTimeFormatter(sc);
	
	final ItemQuery<ItemList> listQuery = ItemList.getQuery();
	listQuery.setIncludes(cc.getInclude());
	listQuery.join(Hql.innerJoin("members", "m"));
	listQuery.restrict(Restrictions.eq(Hql.property("memberType"), Expressions.integer(itemType.getValue())));
	listQuery.restrict(Restrictions.eq(Hql.alias("m"), Expressions.integer(itemId)));
	listQuery.order(Orders.asc(Hql.property("name")));

	ItemResultList<ItemList> lists = listQuery.list(dc);
	long count = lists.getTotalCount();
	%>
	<base:page type="include">
	<base:body>
		<base:section 
			id="itemlists" 
			title="<%=title + " (" + (count) +")"%>"
			context="<%=cc%>"
			>
			<%
			if (count == 0)
			{
				%>
				<div class="messagecontainer note">
				This <%=itemType.toString().toLowerCase() %> is not member in
				any list (or, you don't have permission to view them).
				</div>
				<%
			}
			else
			{
				%>
				<tbl:table
					id="links"
					columns="all"
					>
					<tbl:columndef 
						id="name"
						title="List"
					/>
					<tbl:columndef 
						id="memberType"
						title="Member type"
					/>
					<tbl:columndef 
						id="owner"
						title="Owner"
					/>
					<tbl:columndef 
						id="lastSync"
						title="Last sync"
					/>
					<tbl:columndef 
						id="size"
						title="Size"
					/>
					<tbl:columndef 
						id="description"
						title="Description"
					/>
					<tbl:data>
						<tbl:headers>
							<tbl:headerrow>
								<tbl:columnheaders />
							</tbl:headerrow>
							<tbl:columnsubtitles />
						</tbl:headers>
						<tbl:rows>
						<%
						for (ItemList list : lists)
						{
							%>
							<tbl:row>
								<tbl:cell column="name"><base:icon 
										image="deleted.png" 
										tooltip="This item has been scheduled for deletion" 
										visible="<%=list.isRemoved()%>"
									/><%=Base.getLinkedName(ID, list, false, true)%></tbl:cell>
								<tbl:cell column="memberType">
									<%
									if (itemType != Item.RAWBIOASSAY)
									{
										%>
										<base:propertyvalue item="<%=list%>" property="itemSubtype" nulltext="<i>any</i>"/>
										<%
									}
									else
									{
										RawDataType rdt = list.getRawDataType();
										%>
										<%=rdt != null ? HTML.encodeTags(rdt.getName()) : "<i>any</i>"%>
										<%
									}
									%>
								</tbl:cell>
								<tbl:cell column="owner"><base:propertyvalue item="<%=list%>" property="owner" /></tbl:cell>
								<tbl:cell column="lastSync">
									<%=list.hasSyncFilters() ? (list.getSyncDate() == null ? "<i>never</i>" : timeFormatter.format(list.getSyncDate())) : "<i>n/a</i>" %>
								</tbl:cell>
								<tbl:cell column="size"><%=list.getSize() %></tbl:cell>
								<tbl:cell column="description"><%=HTML.niceFormat(list.getDescription())%></tbl:cell>
							</tbl:row>
							<%
						}
						%>
						</tbl:rows>
					</tbl:data>
				</tbl:table>
				<%
			}
			%>
		</base:section>
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}

%>
