/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var TestWithFile = function()
{
	var test = {};

	var hasExcelSheetParameter = false;
	var currentExcelSheet;
	
	test.PRE_QUOTE = '(?!")';
	test.POST_QUOTE = '(?=")';
	test.WHITESPACE_OR_QUOTE = '[\\s\\"]*';
	test.NOT_INSIDE_QUOTE = '(?=(?:[^"]*"[^"]*")*(?![^"]*"))';
	
	/**
		Initialize the page.
	*/
	test.initPage = function()
	{
		// Buttons
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnSave', test.save);
		Buttons.addClickHandler('btnBrowse', test.selectFile);
		Events.addEventHandler('path', 'base-selected', test.setFileCallback);
		Buttons.addClickHandler('btnParse', test.parseFile);

		Events.addEventHandler('sheet', 'change', test.autoParse);
		Events.addEventHandler('maxLines', 'keypress', Events.integerOnly)
		Events.addEventHandler('maxDataColumns', 'keypress', Events.integerOnly)
		Events.addEventHandler('minDataColumns', 'keypress', Events.integerOnly)

		Buttons.addClickHandler('btnHeaderRegexp', test.showPredefined);
		Buttons.addClickHandler('btnSplitterRegexp', test.showPredefined);
		Buttons.addClickHandler('btnIgnoreRegexp', test.showPredefined);
		
		test.initParameters();
		var frm = document.forms['testwithfile'];
		test.getFileInfo(Strings.trim(frm.path.value));
	}
	
	test.initElements = function(element, autoInit)
	{
		if (autoInit == 'set-preset')
		{
			Events.addEventHandler(element, 'click', test.presetOnClick);
		}
	}
	
	test.selectFile = function()
	{
		var url = '&title=Test+with+file';
		Dialogs.selectItem('FILE', 'path', 0, url);
	}

	test.setFileCallback = function(event)
	{
		var frm = document.forms['testwithfile'];
		frm.file_id.value = event.detail.id;
		frm.path.value = event.detail.name;
		test.getFileInfo(event.detail.name, event.detail.id);
	}
	
	test.getFileInfo = function(path, fileId)
	{
		var request = Ajax.getXmlHttpRequest();
		if (request != null)
		{
			var url = App.getRoot()+'filemanager/files/ajax.jsp?ID='+App.getSessionId();
			url += '&cmd=GetFileInfo&checktype=1';
			if (fileId) 
			{
				url += '&item_id=' + fileId;
			}
			else if (path)
			{
				url += '&path=' + encodeURIComponent(path);
			}
			else
			{
				return false;
			}
			var frm = document.forms['testwithfile'];
			Doc.addClass('btnParse', 'disabled');
			frm.sheet.disabled = true;

			request.open("GET", url, true);
			Ajax.setReadyStateHandler(request, test.fileInfoCallback);
			request.send(null);
		}
		return request != null;
	}
	
	test.fileInfoCallback = function(request)
	{
		var frm = document.forms['testwithfile'];
		Doc.removeClass('btnParse', 'disabled');
		
		var response = JSON.parse(request.responseText);
		var excelMode = response.excelMode;

		frm.filemode.value = excelMode ? 'excel' : 'csv';
		Data.set(document.body, 'filemode', frm.filemode.value);

		frm.sheet.length = 0;
		frm.sheet.disabled = !excelMode;
		frm.charset.disabled = excelMode;
		frm.dataSplitter.disabled = excelMode;
		Doc.addOrRemoveClass('btnSplitterRegexp', 'disabled', excelMode);
		frm.trimQuotes.disabled = excelMode;
		if (excelMode)
		{
			frm.dataSplitter.value = '\\t';
			Forms.selectListOption(frm.charset, 'UTF-8');
			frm.trimQuotes.checked = false;
			for (var i = 0; i < response.sheets.length; i++)
			{
				var sheet = response.sheets[i];
				frm.sheet[frm.sheet.length] = new Option((i+1) + ': '+sheet, sheet, false, currentExcelSheet == sheet);
			}
			if (!hasExcelSheetParameter && frm.sheet.length > 1)
			{
				Doc.show('noSheetSupportWarning', 'inline');
			}
			else
			{
				Doc.hide('noSheetSupportWarning');
			}
		}
		else
		{
			if (charset)
			{
				var charset = response.characterSet;
				Forms.selectListOption(frm.charset, charset);
			}
			frm.trimQuotes.checked = true;
		}
		test.autoParse();
	}
	
	// Parse the selected file if we have the required parameters
	test.autoParse = function()
	{
		var frm = document.forms['testwithfile'];
		if (Strings.trim(frm.path.value) != '' && Strings.trim(frm.dataSplitter.value) != '')
		{
			test.parseFile();
		}
	}
	
	test.parseFile = function()
	{
		var frm = document.forms['testwithfile'];
		if (Strings.trim(frm.path.value) == '')
		{
			Forms.showNotification('btnBrowse', 'You must select a file');
			return false;
		}
		else if (Strings.trim(frm.dataSplitter.value) == '')
		{
			Forms.showNotification(frm.dataSplitter, 'You must specify a data splitter regular expression');
			return false;
		}
		
		var parseWin = window.frames['parsed'];
		if (parseWin && parseWin.ParseFile)
		{
			parseWin.ParseFile.pleaseWaitWhileParsing();
		}
		frm.submit();
	}
	
	test.initParameters = function()
	{
		var frm = document.forms['testwithfile'];
		var configWin = window.opener;
		frm.path.value = configWin.Configure.getParameterValue('file');
		frm.header.value = configWin.Configure.getParameterValue('headerRegexp');
		frm.dataHeader.value = configWin.Configure.getParameterValue('dataHeaderRegexp');
		frm.dataSplitter.value = configWin.Configure.getParameterValue('dataSplitterRegexp');
		frm.dataFooter.value = configWin.Configure.getParameterValue('dataFooterRegexp');
		frm.ignore.value = configWin.Configure.getParameterValue('ignoreRegexp');
		frm.minDataColumns.value = configWin.Configure.getParameterValue('minDataColumns');
		frm.maxDataColumns.value = configWin.Configure.getParameterValue('maxDataColumns');
		frm.trimQuotes.checked = configWin.Configure.getParameterValue('trimQuotes') != 'false';
		frm.decimalSeparator.value = configWin.Configure.getParameterValue('decimalSeparator');
		frm.dateFormat.value = configWin.Configure.getParameterValue('dateFormat');
		frm.timestampFormat.value = configWin.Configure.getParameterValue('timestampFormat');
		hasExcelSheetParameter = configWin.Configure.getParameterDef('excelSheet') != null;
		currentExcelSheet = configWin.Configure.getParameterValue('excelSheet');
		
		// Copy the list since ith is not certain that it is the same as Config.allCharsets() 
		var allCharsets = configWin.Doc.element('parameter-charset');
		if (allCharsets)
		{
			frm.charset.length = 0;
			for (var i = 0; i < allCharsets.length; i++)
			{
				var cs = allCharsets[i];
				if (cs.value != '') frm.charset[frm.charset.length] = new Option(cs.text, cs.value);
			}
		}
		Forms.selectListOption(frm.charset, configWin.Configure.getParameterValue('charset'));
		
		var allParameters = configWin.Configure.getAllParameters();
		var parameterNames = [];
		for (var i = 0; i < allParameters.length; i++)
		{
			var param = allParameters[i];
			if (param.name.indexOf('Mapping') >= 0)
			{
				parameterNames[parameterNames.length] = param.name;
				Forms.addHidden(frm, 'mapping.'+param.name+'.label', param.label);
				Forms.addHidden(frm, 'mapping.'+param.name+'.expression', param.values.length == 0 ? '' : param.values[0]);
			}
		}
		if (parameterNames.length > 0)
		{
			Forms.addHidden(frm, 'mappingParameterNames', parameterNames.join(','));
		}
	}
	
	test.save = function()
	{
		var frm = document.forms['testwithfile'];
		var configWin = window.opener;
		configWin.Configure.setParameterValue('file', frm.path.value);
		configWin.Configure.setParameterValue('headerRegexp', frm.header.value);
		configWin.Configure.setParameterValue('dataHeaderRegexp', frm.dataHeader.value);
		configWin.Configure.setParameterValue('dataSplitterRegexp', frm.dataSplitter.value);
		configWin.Configure.setParameterValue('dataFooterRegexp', frm.dataFooter.value);
		configWin.Configure.setParameterValue('ignoreRegexp', frm.ignore.value);
		configWin.Configure.setParameterValue('minDataColumns', frm.minDataColumns.value);
		configWin.Configure.setParameterValue('maxDataColumns', frm.maxDataColumns.value);
		configWin.Configure.setParameterValue('trimQuotes', frm.trimQuotes.checked ? 'true' : 'false');
		configWin.Configure.setParameterValue('charset', frm.charset.value);
		configWin.Configure.setParameterValue('excelSheet', frm.sheet.value);
		
		var allParameters = configWin.Configure.getAllParameters();
		for (var i = 0; i < allParameters.length; i++)
		{
			var name = allParameters[i].name;
			if (name.indexOf('Mapping') >= 0)
			{
				configWin.Configure.setParameterValue(name, frm['mapping.'+name+'.expression'].value);
			}
		}
		window.close();
	}
	
	test.showPredefined = function(event)
	{
		var menu = Data.get(event.currentTarget, 'predefined');
		Menu.toggleTopMenu(menu, event.clientX, event.clientY); 
		event.stopPropagation();
	}
	
	test.presetOnClick = function(event)
	{
		var field = Data.get(event.currentTarget, 'field');
		var value = Data.get(event.currentTarget, 'value');
		test.setFormValue(field, value);
	}
	
	test.setFormValue = function(field, value)
	{
		var frm = document.forms['testwithfile'];
		frm[field].value = value;
	}

	return test;
}();

Doc.onLoad(TestWithFile.initPage);
Doc.addElementInitializer(TestWithFile.initElements);
