/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Login = function()
{
	var pUseLastLogin = 1;
	
	var login = {};
	
	login.initPage = function()
	{
		// If there is no requested login form, we try to use the last one that was used
		var requestedForm = Data.get(document.body, 'requested-form');
		if (!requestedForm)
		{
			var lastLoginForm = login.getLastLoginForm();
			var selectedLoginForm = Data.get(document.body, 'login-form');
			if (lastLoginForm && selectedLoginForm && lastLoginForm != selectedLoginForm)
			{
				login.switchLoginForm(lastLoginForm);
				return;
			}
		}
		
		// Information links
		Buttons.addClickHandler('aboutServer', login.showAbout);
		Buttons.addClickHandler('getAccount', login.showGetAccount);
		Buttons.addClickHandler('forgotPassword', login.showForgotPassword);
		
		// Login buttons
		Events.doOnEnter('login', login.setFocus);
		Buttons.addClickHandler('btnLogin', login.doLogin);
		Buttons.addClickHandler('btnLoginAnyway', login.enableLogin);
		Buttons.addClickHandler('close', App.closeWindow);
		
		Events.addEventHandler('loginForm', 'change', login.loginFormOnChange);
		
		var extraField = Doc.element('extraField');
		if (extraField == null)
		{
			// Password field should login if 'ENTER' is pressed
			Events.doOnEnter('password', login.doLogin);
		}
		else
		{
			// Password field should move focus to the 'extraField' if 'ENTER' is pressed
			Events.doOnEnter('password', login.setFocus);
			Events.doOnEnter(extraField, login.doLogin);
		}
		
		Doc.show('the-login-form', 'table');
		login.initForm();
		
		var footNoteFrame = App.topWindow().frames['footnote'];
		if (footNoteFrame)
		{
			setTimeout(login.reloadFootnote, 500);
		}
	}
	
	
	login.loginFormOnChange = function(event)
	{
		login.switchLoginForm(event.currentTarget.value);
	}
	
	login.switchLoginForm = function(loginForm)
	{
		var frm = document.forms['login'];
		var url = location.pathname+'?ID='+App.getSessionId();
		url += '&loginForm='+encodeURIComponent(loginForm);
		if (frm.again) url += '&again='+encodeURIComponent(frm.again.value);
		location.replace(url);
	}
	
	login.reloadFootnote = function()
	{
		var footNoteFrame = App.topWindow().frames['footnote'];
		footNoteFrame.location.reload();			
	}
	
	/**
		Initialize the login form by populating it with the last used login
		and setting focus to either the login or password field.
	*/
	login.initForm = function()
	{
		var frm = document.forms['login'];
		if (!frm.login) return;
		pUseLastLogin = Data.int(frm.login, 'use-last-login', 1);
		if (pUseLastLogin)
		{
			var lastLogin = login.getLastLogin();
			if (!frm.login.value && lastLogin) 
			{
				frm.login.value = lastLogin;
			}
		}
		login.setFocus();
	}

	
	/*
		Save the last login. If local storage is available we use that,
		otherwise we simply save it on the top frameset.
	*/
	login.saveLastLogin = function(lastLogin)
	{
		var storage = App.localStorage();
		if (storage)
		{
			App.setLocal('last-login', lastLogin);
		}
		else
		{
			App.topWindow().lastLogin = lastLogin;
		}
	}
	
	/**
		Get the last used login. If local storage is available we
		use that, otherwise we use the top frameset.
	 */
	login.getLastLogin = function()
	{
		var lastLogin;
		var storage = App.localStorage();
		if (storage)
		{
			lastLogin = App.getLocal('last-login');
		}
		if (!lastLogin)
		{
			lastLogin = App.topWindow().lastLogin;
		}
		return lastLogin;
	}
	
	
	/*
		Save the last login form. If local storage is available we use that,
		otherwise we simply save it on the top frameset.
	*/
	login.saveLastLoginForm = function(lastLoginForm)
	{
		var storage = App.localStorage();
		if (storage)
		{
			App.setLocal('last-login-form', lastLoginForm);
		}
		else
		{
			App.topWindow().lastLoginForm = lastLoginForm;
		}
	}
	
	/**
		Get the last used login form. If local storage is available we
		use that, otherwise we use the top frameset.
	 */
	login.getLastLoginForm = function()
	{
		var lastLoginForm;
		var storage = App.localStorage();
		if (storage)
		{
			lastLoginForm = App.getLocal('last-login-form');
		}
		if (!lastLoginForm)
		{
			lastLoginForm = App.topWindow().lastLoginForm;
		}
		return lastLoginForm;
	}

	
	/**
		Set focus on the login field if empty, otherwise the password field 
		or the extraField
	*/
	login.setFocus = function()
	{
		var frm = document.forms['login'];
		var fieldToFocus = null;
		if (!frm.login.value)
		{
			fieldToFocus = frm.login;
		}
		else if (frm.password.value && frm.extraField)
		{
			fieldToFocus = frm.extraField;
		}
		else
		{
			fieldToFocus = frm.password;
		}
		fieldToFocus.focus();
		fieldToFocus.select();
	}
	
	/**
		Enable the (disabled) login form.
	*/
	login.enableLogin = function()
	{
		var frm = document.forms['login'];
		frm.login.disabled = false;
		frm.password.disabled = false;
		if (frm.extraField) frm.extraField.disabled = false;
		Doc.hide('btnLoginAnyway');
		Doc.removeClass('btnLogin', 'disabled');
		login.setFocus();
	}
	
	/**
		Send the login form.
	*/
	login.doLogin = function(event)
	{
		var frm = document.forms['login'];
		// Check if login has been disabled
		if (frm.login && frm.login.disabled) return;

		// Disable start page extension if special key is used
		var specialKey = event.altKey || event.ctrlKey || event.shiftKey;
		
		// Check if the login is not recommended
		var denyLogin = Doc.element('denyLogin');
		if (denyLogin)
		{
			if (!confirm('Login has been disabled by an administrator. Do you want to continue anyway?'))
			{
				return;
			}
		}
		
		// On the impersonate form, check that a user has been selected
		if (frm.user_id && !frm.user_id.value)
		{
			Forms.showNotification('user_id.select', 'Please select a user');
			return;
		}

		// Store the last login
		if (pUseLastLogin && frm.login)
		{
			Login.saveLastLogin(frm.login.value);
		}
		if (frm.loginForm)
		{
			Login.saveLastLoginForm(frm.loginForm.value);
		}
		
		// Check 'remain on page' and 'redirect' options
		if (frm.remainOnPage && frm.redirect)
		{
			if (frm.remainOnPage.checked) 
			{
				frm.redirect.value = window.opener ? window.opener.location.href : '';
			}
			else
			{
				frm.useAutoStartPage.value = '1';
				frm.redirect.value = App.getRoot()+'my_base/index.jsp?ID='+App.getSessionId();
			}
		}
		
		if (specialKey && frm.useAutoStartPage) frm.useAutoStartPage.value = '0'; 
		
		if (frm.target) Dialogs.openPopup('', frm.target, 300, 200);
		// Set the deviceToken if we have it in local storage
		if (frm.deviceToken)
		{
			var deviceToken = App.getLocal('deviceToken');
			if (deviceToken) 
			{
				frm.deviceToken.value = deviceToken;
			}
			else
			{
				// Otherwise we save the auto-generated value
				App.setLocal('deviceToken', frm.deviceToken.value);
			}
		}
		if (!pUseLastLogin)
		{
			// Duplicate in hidden form
			var frm2 = Forms.cloneAsHidden(frm);
			document.body.appendChild(frm2);
			frm2.submit();
			document.body.removeChild(frm2);
		}
		else
		{
			frm.submit();
		}
		
	}
	
	/**
		Show popup dialog with information about this server.
	*/
	login.showAbout = function()
	{
		var url = App.getRoot()+'info/about.jsp?ID='+App.getSessionId();
		url += '&page=about';
		Dialogs.openPopup(url, 'About', 600, 400);
	}
	
	/**
		Show popup dialog with information about how to get an account on this server.
	*/
	login.showGetAccount = function()
	{
		var url = App.getRoot()+'info/get_account.jsp?ID='+App.getSessionId();
		Dialogs.openPopup(url, 'GetAccount', 450, 300);
	}
	
	/**
		Show popup dialog with information about what to do if a password has been lost.
	*/
	login.showForgotPassword = function()
	{
		var url = App.getRoot()+'info/forgot_password.jsp?ID='+App.getSessionId();
		Dialogs.openPopup(url, 'ForgotPassword', 450, 300);
	}
	
	return login;
}();

Doc.onLoad(Login.initPage);

